# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.2.0

import winrt.system
from winrt._winrt_windows_management_deployment_preview import (
    ClassicAppManager,
    InstalledClassicAppInfo,
)

__all__ = [
    "ClassicAppManager",
    "InstalledClassicAppInfo",
]

